// ==============================================================
//
// maven.cpp
// Control module for MAVEN vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

const double ISP = 1975;
const double MAINTH = 1020;
const double RCS6TH = 22;
const double RCS8TH = 1;
const double FUELMASS = 1647;
const double DRYMASS = 903;

const double APP_ARM_SPEED = 0.2;
const double APP_AZ_SPEED = 0.2;
const double APP_ELEV_SPEED = 0.2;
const double TOP_ARM_SPEED = 0.2;
const double SOLP_SPEED = 0.05;
const double ANT_SPEED = 0.05;

#define GREEN RGB(125, 255, 125)

void VLiftCoeff(VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd) {
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = { -180 * RAD,-60 * RAD,-30 * RAD, -2 * RAD, 10 * RAD,15 * RAD,50 * RAD,90 * RAD,180 * RAD };
	static const double CL[nabsc] = { 0,0,0,0,0,0,0,0,0 };
	static const double CM[nabsc] = { 0,0,0,0,0,0,0,0,0 };
	for (i = 0; i < nabsc - 1 && AOA[i + 1] < aoa; i++);
	double f = (aoa - AOA[i]) / (AOA[i + 1] - AOA[i]);
	*cl = CL[i] + (CL[i + 1] - CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i + 1] - CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.03 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag(*cl, 1.5, 0.7) + oapiGetWaveDrag(M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

void HLiftCoeff(VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd) {
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = { -180 * RAD,-135 * RAD,-90 * RAD,-45 * RAD,45 * RAD,90 * RAD,135 * RAD,180 * RAD };
	static const double CL[nabsc] = { 0,0,0,0,0,0,0,0 };
	for (i = 0; i < nabsc - 1 && BETA[i + 1] < beta; i++);
	*cl = CL[i] + (CL[i + 1] - CL[i]) * (beta - BETA[i]) / (BETA[i + 1] - BETA[i]);
	*cm = 0.0;
	*cd = 0.03 + oapiGetInducedDrag(*cl, 1.5, 0.7) + oapiGetWaveDrag(M, 0.75, 1.0, 1.1, 0.04);
}
// ==============================================================
// MAVEN class interface
// ==============================================================

class maven : public VESSEL3 {
public:
	maven(OBJHANDLE hVessel, int flightmodel);
	~maven();
	void clbkSetClassCaps(FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkSaveState(FILEHANDLE scn);
	void clbkLoadStateEx(FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkPostCreation();
	void DefineAnimations(void);

	THRUSTER_HANDLE th_rcs[15], th_group[6];
	PROPELLANT_HANDLE hProp;

	ANIMATIONCOMPONENT_HANDLE Section, Section1, Section2, Section3, Section4, Section5;
private:
	double step_time;
	int kr_status, solp_status, apparm_status, appaz_status;
	int appelev_status, toparm_status, ant_status;

	UINT anim_solp, anim_apparm, anim_appaz, anim_appelev, anim_toparm, anim_ant;
	double solp_proc, apparm_proc, appaz_proc, appelev_proc, toparm_proc, ant_proc;
};

maven::maven (OBJHANDLE hVessel, int flightmodel) : VESSEL3 (hVessel, flightmodel) {}

maven::~maven (){}

void maven::DefineAnimations(void) {
	// 1. SOLP
	anim_solp = CreateAnimation(0);
	static UINT Solp1Grp[1] = { 13 };
	static MGROUP_ROTATE Solp1(0, Solp1Grp, 1, _V(-1.27, 0, 1.051), _V(0, 1, 0), (float)(90 * RAD));

	static UINT Solp2Grp[1] = { 14 };
	static MGROUP_ROTATE Solp2(0, Solp2Grp, 1, _V(-1.296, 0, -0.894), _V(0, 1, 0), (float)(-165 * RAD));

	static UINT Solp3Grp[1] = { 15 };
	static MGROUP_ROTATE Solp3(0, Solp3Grp, 1, _V(1.27, 0, 1.051), _V(0, 1, 0), (float)(-90 * RAD));

	static UINT Solp4Grp[1] = { 16 };
	static MGROUP_ROTATE Solp4(0, Solp4Grp, 1, _V(1.296, 0, -0.894), _V(0, 1, 0), (float)(165 * RAD));

	Section = AddAnimationComponent(anim_solp, 0.5, 1, &Solp1);
	AddAnimationComponent(anim_solp, 0, 0.5, &Solp2, Section);

	Section1 = AddAnimationComponent(anim_solp, 0.5, 1, &Solp3);
	AddAnimationComponent(anim_solp, 0, 0.5, &Solp4, Section1);

	// App Arm
	anim_apparm = CreateAnimation(0);
	static UINT AppArm1Grp[1] = { 6 };
	static MGROUP_ROTATE AppArm1(0, AppArm1Grp, 1, _V(0, -1.326, 0.961), _V(1, 0, 0), (float)(-98 * RAD));

	static UINT AppArm2Grp[4] = { 2,3,4,5 };
	static MGROUP_ROTATE AppArm2(0, AppArm2Grp, 4, _V(0, -1.167, -0.174), _V(1, 0, 0), (float)(98 * RAD));

	Section2 = AddAnimationComponent(anim_apparm, 0, 1, &AppArm1);
	AddAnimationComponent(anim_apparm, 0, 1, &AppArm2, Section2);

	// top arm
	anim_toparm = CreateAnimation(0);
	static UINT TopArm1Grp[2] = { 17,18 };
	static MGROUP_ROTATE TopArm1(0, TopArm1Grp, 2, _V(0, 1.341, -0.793), _V(1, 0, 0), (float)(-150 * RAD));

	AddAnimationComponent(anim_toparm, 0, 1, &TopArm1);

	// ants
	anim_ant = CreateAnimation(0);
	static UINT Ant1Grp[1] = { 0 };
	static MGROUP_SCALE Ant1(0, Ant1Grp, 1, _V(-0.549, 1.305, -0.511), _V(0.01, 0.01, 0.01));

	static UINT Ant2Grp[1] = { 1 };
	static MGROUP_SCALE Ant2(0, Ant2Grp, 1, _V(0.549, 1.305, -0.511), _V(0.01, 0.01, 0.01));

	AddAnimationComponent(anim_ant, 0, 1, &Ant1);
	AddAnimationComponent(anim_ant, 0, 1, &Ant2);
}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void maven::clbkSetClassCaps(FILEHANDLE cfg) {
	step_time = 0;
	kr_status = 0;

	solp_status = 0;
	apparm_status = 0;
	appaz_status = 0;
	appelev_status = 0;
	toparm_status = 0;
	ant_status = 2;

	solp_proc = 0.0;
	apparm_proc = 0.0;
	appaz_proc = 0.0;
	appelev_proc = 0.0;
	toparm_proc = 0.0;
	ant_proc = 1.0;

	DefineAnimations();
	// physical vessel parameters
	SetSize(2);
	SetEmptyMass(DRYMASS);
	SetPMI(_V(1.9, 2.2, 2.3));
	SetCrossSections(_V(8, 8, 8));
	SetCW(1, 1, 1, 1);
	SetRotDrag(_V(2, 2, 1));
	SetTouchdownPoints(_V(0, -1, 5), _V(-2, -1, -5), _V(2, -1, -5));

	// camera parameters
	SetCameraOffset(_V(0, 1.5, -1));

	SetMeshVisibilityMode(AddMesh("maven/maven"), MESHVIS_ALWAYS);

	// propellant resources
	hProp = CreatePropellantResource(FUELMASS, FUELMASS);

	// ***************** thruster definitions *******************

	// main
	th_rcs[0] = CreateThruster(_V(0, 0, -1.75), _V(0, 0, 1), MAINTH, hProp, ISP);

	// rcs6
	th_rcs[1] = CreateThruster(_V(1.162, 1.366, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);
	th_rcs[2] = CreateThruster(_V(-1.162, 1.366, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);

	th_rcs[3] = CreateThruster(_V(1.162, 0, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);
	th_rcs[4] = CreateThruster(_V(-1.162, 0, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);

	th_rcs[5] = CreateThruster(_V(1.162, -1.366, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);
	th_rcs[6] = CreateThruster(_V(-1.162, -1.366, -0.86), _V(0, 0, 1), RCS6TH, hProp, ISP);

	// rcs8
	th_rcs[7] = CreateThruster(_V(1.285, 1.411, 0.51), _V(-0.9659, -0.2588, 0), RCS6TH, hProp, ISP);
	th_rcs[8] = CreateThruster(_V(1.285, 1.411, -0.744), _V(-0.9659, -0.2588, 0), RCS6TH, hProp, ISP);

	th_rcs[9] = CreateThruster(_V(-1.285, 1.411, 0.51), _V(0.9659, -0.2588, 0), RCS6TH, hProp, ISP);
	th_rcs[10] = CreateThruster(_V(-1.285, 1.411, -0.744), _V(0.9659, -0.2588, 0), RCS6TH, hProp, ISP);

	th_rcs[11] = CreateThruster(_V(1.285, -1.411, 0.51), _V(-0.9659, 0.2588, 0), RCS6TH, hProp, ISP);
	th_rcs[12] = CreateThruster(_V(1.285, -1.411, -0.744), _V(-0.9659, 0.2588, 0), RCS6TH, hProp, ISP);

	th_rcs[13] = CreateThruster(_V(-1.285, -1.411, 0.51), _V(0.9659, 0.2588, 0), RCS6TH, hProp, ISP);
	th_rcs[14] = CreateThruster(_V(-1.285, -1.411, -0.744), _V(0.9659, 0.2588, 0), RCS6TH, hProp, ISP);


	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	CreateThrusterGroup(th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup(th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[2];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[6];
	CreateThrusterGroup(th_group, 3, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[5];
	CreateThrusterGroup(th_group, 3, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	th_group[2] = th_rcs[11];
	th_group[3] = th_rcs[12];
	CreateThrusterGroup(th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[7];
	th_group[1] = th_rcs[8];
	th_group[2] = th_rcs[13];
	th_group[3] = th_rcs[14];
	CreateThrusterGroup(th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[4];
	th_group[4] = th_rcs[5];
	th_group[5] = th_rcs[6];
	CreateThrusterGroup(th_group, 6, THGROUP_ATT_FORWARD);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	th_group[2] = th_rcs[13];
	th_group[3] = th_rcs[14];
	CreateThrusterGroup(th_group, 4, THGROUP_ATT_RIGHT);

	th_group[0] = th_rcs[7];
	th_group[1] = th_rcs[8];
	th_group[2] = th_rcs[11];
	th_group[3] = th_rcs[12];
	CreateThrusterGroup(th_group, 4, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[0];
	CreateThrusterGroup(th_group, 1, THGROUP_MAIN);

	SURFHANDLE tex_main = oapiRegisterExhaustTexture("Exhaust2");
	SURFHANDLE exhaust = oapiRegisterExhaustTexture("exhaust_atrcs");

	double exmain_len = 1.5;
	double exmain_wid = 0.15;

	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(0, 0.155, -1.85), _V(0, 0, -1), tex_main);
	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(0, -0.155, -1.85), _V(0, 0, -1), tex_main);

	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(0.133, 0.077, -1.85), _V(0, 0, -1), tex_main);
	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(-0.133, 0.077, -1.85), _V(0, 0, -1), tex_main);

	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(0.133, -0.077, -1.85), _V(0, 0, -1), tex_main);
	AddExhaust(th_rcs[0], exmain_len, exmain_wid, _V(-0.133, -0.077, -1.85), _V(0, 0, -1), tex_main);

	double exlen = 1;
	double exwid = 0.1;

	AddExhaust(th_rcs[1], exlen, exwid, exhaust);
	AddExhaust(th_rcs[2], exlen, exwid, exhaust);
	AddExhaust(th_rcs[3], exlen, exwid, exhaust);
	AddExhaust(th_rcs[4], exlen, exwid, exhaust);
	AddExhaust(th_rcs[5], exlen, exwid, exhaust);
	AddExhaust(th_rcs[6], exlen, exwid, exhaust);

	double exlen2 = 0.5;
	double exwid2 = 0.05;

	AddExhaust(th_rcs[7], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[8], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[9], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[10], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[11], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[12], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[13], exlen2, exwid2, exhaust);
	AddExhaust(th_rcs[14], exlen2, exwid2, exhaust);

	// AirFoils
	CreateAirfoil3(LIFT_VERTICAL, _V(0, 0, 10), VLiftCoeff, 0, 1, 2, 1);
	CreateAirfoil3(LIFT_HORIZONTAL, _V(0, 0, 10), HLiftCoeff, 0, 1, 2, 1);
}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel) { return new maven(hvessel, flightmodel); }

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit(VESSEL *vessel) { if (vessel) delete (maven*)vessel; }




void maven::clbkSaveState(FILEHANDLE scn) {
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf(cbuf, "%d %0.4f %d %0.4f", solp_status, solp_proc, ant_status, ant_proc);
	oapiWriteScenario_string(scn, "SOLP", cbuf);

	sprintf(cbuf, "%d %0.4f %d %0.4f", apparm_status, apparm_proc, toparm_status, toparm_proc);
	oapiWriteScenario_string(scn, "APPA", cbuf);

	sprintf(cbuf, "%d", kr_status);
	oapiWriteScenario_string(scn, "KROT", cbuf);
}
void maven::clbkLoadStateEx(FILEHANDLE scn, void *vs) {
	char *line;
	while (oapiReadScenario_nextline(scn, line)) {
		if (!strnicmp(line, "SOLP", 4)) {
			sscanf(line + 4, "%d%lf%d%lf", &solp_status, &solp_proc, &ant_status, &ant_proc);
		}
		else if (!strnicmp(line, "APPA", 4)) {
			sscanf(line + 4, "%d%lf%d%lf", &apparm_status, &apparm_proc, &toparm_status, &toparm_proc);
		}
		else if (!strnicmp(line, "KROT", 4)) {
			sscanf(line + 4, "%d", &kr_status);
		}
		else {
			ParseScenarioLineEx(line, vs);
			// unrecognised option - pass to Orbiter's generic parser
		}
	}

	SetAnimation(anim_solp, solp_proc);
	SetAnimation(anim_ant, ant_proc);
	SetAnimation(anim_apparm, apparm_proc);
	SetAnimation(anim_toparm, toparm_proc);

	if (solp_status == 3) {
		SetPMI(_V(1.9, 2.2, 2.3));
		SetCrossSections(_V(8, 8, 8));
	}
}

int maven::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	if (!down) return 0; // only process keydown events

	if (KEYMOD_SHIFT(kstate))
	{
		switch (key)
		{


		case OAPI_KEY_NUMPAD5: // killrot
			if (kr_status == 0) kr_status = 1;
			else if (kr_status == 1) kr_status = 0;
			return 1;

		}
	}


	else if (KEYMOD_CONTROL(kstate)){}

	else { // unmodified keys
		switch (key) {

		case OAPI_KEY_G: // solps
			if (solp_status == 0) solp_status = 1;
			return 1;

		case OAPI_KEY_K: // top arm
			if (toparm_status == 0) toparm_status = 1;
			return 1;

		case OAPI_KEY_V: // ants
			if (ant_status == 2) ant_status = 1;
			return 1;

		case OAPI_KEY_B: // apparm
			if (apparm_status == 0) apparm_status = 1;
			return 1;


		}
	}
	return 0;
}


int maven::clbkConsumeDirectKey(char *kstate) {
	VECTOR3 avel = _V(0, 0, 0);
	GetAngularVel(avel);
	double dt = oapiGetSimStep();
	double drot = RAD * 2 * dt;

	if (kr_status == 0)
	{
		if (KEYMOD_SHIFT(kstate))
		{
			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD2))
			{
				if (avel.x < RAD * 10)
				{
					avel.x = avel.x + drot;
					if (avel.x > RAD * 10) avel.x = RAD * 10;
				}
			}

			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD8))
			{
				if (avel.x > -RAD * 10)
				{
					avel.x = avel.x - drot;
					if (avel.x < -RAD * 10) avel.x = -RAD * 10;
				}
			}

			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD1))
			{
				if (avel.y < RAD * 10)
				{
					avel.y = avel.y + drot;
					if (avel.y > RAD * 10) avel.y = RAD * 10;
				}
			}

			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD3))
			{
				if (avel.y > -RAD * 10)
				{
					avel.y = avel.y - drot;
					if (avel.y < -RAD * 10) avel.y = -RAD * 10;
				}
			}

			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD6))
			{
				if (avel.z < RAD * 10)
				{
					avel.z = avel.z + drot;
					if (avel.z > RAD * 10) avel.z = RAD * 10;
				}
			}

			if (KEYDOWN(kstate, OAPI_KEY_NUMPAD4))
			{
				if (avel.z > -RAD * 10)
				{
					avel.z = avel.z - drot;
					if (avel.z < -RAD * 10) avel.z = -RAD * 10;
				}
			}

			SetAngularVel(avel);
		}
	}

	return 0;
}


void maven::clbkPostCreation() { if (ant_status == 2) SetAnimation(anim_ant, ant_proc); }



void maven::clbkPreStep(double SimT, double SimDT, double MJD) {

	step_time = SimDT;
	double simdt = SimDT;

	if (kr_status == 1) {
		VECTOR3 avel = _V(0, 0, 0);
		GetAngularVel(avel);
		double drot = RAD * 2 * simdt;

		if (avel.x > -drot && avel.x < drot) avel.x = 0;
		else if (avel.x > drot) avel.x = avel.x - drot;
		else if (avel.x < -drot) avel.x = avel.x + drot;

		if (avel.y > -drot && avel.y < drot) avel.y = 0;
		else if (avel.y > drot) avel.y = avel.y - drot;
		else if (avel.y < -drot) avel.y = avel.y + drot;

		if (avel.z > -drot && avel.z < drot) avel.z = 0;
		else if (avel.z > drot) avel.z = avel.z - drot;
		else if (avel.z < -drot) avel.z = avel.z + drot;

		if ((avel.x + avel.y + avel.z) == 0)kr_status = 0;

		SetAngularVel(avel);
	}

	////////////////// ANIMATIONS ///////////////
	// solp
	if (solp_status > 0) {
		double da = SimDT * SOLP_SPEED;

		solp_proc = solp_proc + da;
		if (solp_proc > 1)
		{
			solp_proc = 1;
			solp_status = 2;
		}
		SetAnimation(anim_solp, solp_proc);
	}

	if (solp_status == 2) {
		SetPMI(_V(1.9, 2.2, 2.3));
		SetCrossSections(_V(8, 8, 8));
		solp_status = 3;
	}


	// apparm
	if (apparm_status > 0)
	{
		double da = SimDT * APP_ARM_SPEED;

		apparm_proc = apparm_proc + da;
		if (apparm_proc > 1)
		{
			apparm_proc = 1;
			apparm_status = 2;
		}
		SetAnimation(anim_apparm, apparm_proc);
	}

	// toparm
	if (toparm_status > 0)
	{
		double da = SimDT * TOP_ARM_SPEED;

		toparm_proc = toparm_proc + da;
		if (toparm_proc > 1)
		{
			toparm_proc = 1;
			toparm_status = 2;
		}
		SetAnimation(anim_toparm, toparm_proc);
	}


	/// ants
	if (ant_status == 1)
	{
		double da = SimDT * ANT_SPEED;

		ant_proc = ant_proc - da;
		if (ant_proc < 0)
		{
			ant_proc = 0;
			ant_status = 0;
		}
		SetAnimation(anim_ant, ant_proc);
	}

	else if (ant_status == 2) SetAnimation(anim_ant, ant_proc);
}

bool maven::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp) {
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;

	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}

	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double test = ((m1+m2)/m1);
	double dv = (ISP*(log(test)));

	sprintf(abuf,"Remaining dV = %0.2f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Reaction Wheel Att.Ctrl. = SHIFT+NUMPAD");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	if (kr_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"RW KillRot Active");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (solp_status == 0)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"G = Deploy Solar Panels");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	if (toparm_status == 0)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"K = Deploy SWEA");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	if (ant_status == 2)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"V = Deploy LPW");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	if (apparm_status == 0)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"B = Deploy APP");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	return true;
}

